#!/usr/bin/env python
import click
from pyperclip import copy


@click.command()
@click.argument("frames", nargs=-1, required=True, type=int)
@click.option("-d", "--denum", type=float, default=2)
def frames_denum(frames: tuple[int], denum: float) -> None:
    """
    Sort frames and divide without remainder by the specified divisor.
    The result will be copied to the clipboard.

    \b
    >>> frames_denum((16886, 26280), denum=2)
    <<< "8443 13140"
    >>> frames_denum((16886, 26280), denum=.5)
    <<< "33772 52560"
    """
    divided = " ".join(f"{int(frame // denum)}" for frame in sorted(frames, key=int))

    click.echo(divided)
    copy(divided)



#!/usr/bin/env python
import click

from encode_utils_cli.chapt2bmqpyml import chapt2bmqpyml
from encode_utils_cli.frames_denum import frames_denum
from encode_utils_cli.mpls2chap import mpls2chap
from encode_utils_cli.num_frames import num_frames
from encode_utils_cli.re_chapters import re_chapters
from encode_utils_cli.re_titles import re_titles
from encode_utils_cli.screens2bm import screens2bm
from encode_utils_cli.vs_screens import vs_screens
from encode_utils_cli.zones_validator import zones_validator


@click.group(context_settings={"show_default": True, "help_option_names": ["-h", "--help"]})
@click.version_option()
def cli() -> None:
    """Main entrypoint."""


cli.add_command(chapt2bmqpyml)
cli.add_command(frames_denum)
cli.add_command(mpls2chap)
cli.add_command(num_frames)
cli.add_command(re_chapters)
cli.add_command(re_titles)
cli.add_command(screens2bm)
cli.add_command(vs_screens)
cli.add_command(zones_validator)



#!/usr/bin/env python
from pathlib import Path
from re import MULTILINE, sub

import click
from pyperclip import copy
from tomli import loads


@click.command()
@click.option(
    "-c",
    "--config",
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="toml config.",
)
def re_titles(config: Path) -> None:
    """
    Replace titles names from anidb.
    The result will be copied to the clipboard.

    \b
    >>> 1 	The Prince`s New Clothes
    <<< e1: EP1 «The Prince`s New Clothes»
    """
    titles = loads(config.read_text())["titles"]

    titles = sub(r"	", r" ", titles)
    titles = sub(r"  (.+?)\n", r" «\1»\n", titles)
    titles = sub(r" »", r"»", titles)
    titles = sub(r"^(\d+) ", r"e\1: EP\1 ", titles, flags=MULTILINE)
    titles = sub(r"^OP(\d+)", r"op\1: OP\1", titles, flags=MULTILINE)
    titles = sub(r"^ED(\d+)", r"ed\1: ED\1", titles, flags=MULTILINE)
    titles = sub(r"^S(\d+) ", r"s\1: S\1 ", titles, flags=MULTILINE)

    click.echo(titles)
    copy(titles)



#!/usr/bin/env python
from fractions import Fraction


def seconds2ts(s: float) -> str:
    """
    Convert seconds in timestamp hh:mm:ss.xxx
    """
    m = s // 60
    s %= 60
    h = m // 60
    m %= 60
    return f"{h:02.0f}:{m:02.0f}:{s:06.3f}"


def ts2seconds(ts: str) -> float:
    """
    Convert timestamp hh:mm:ss.xxxx to seconds
    """
    h, m, s = map(float, ts.split(":"))
    return h * 3600 + m * 60 + s


def seconds2f(s: float, fps: Fraction) -> int:
    """
    Convert seconds to frames
    """
    return round(s * fps)


def ts2f(ts: str, fps: Fraction) -> int:
    """
    Convert a timestamp hh:mm:ss.xxxx in number of frames
    """
    return seconds2f(s=ts2seconds(ts), fps=fps)



#!/usr/bin/env python
from pathlib import Path

import click

from encode_utils_cli.util.load_mpls import load_mpls
from encode_utils_cli.util.timeconv import seconds2ts


@click.command()
@click.argument(
    "mpls",
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
@click.option(
    "-s",
    "--start-ep",
    type=int,
    default=1,
    help="Number of first episode in mpls.",
)
@click.option(
    "-d",
    "--out-dir",
    type=click.Path(file_okay=False, path_type=Path),
    help="Custom out dir.",
)
def mpls2chap(mpls: Path, start_ep: int, out_dir: Path) -> None:
    """
    Make .txt chapters from .mpls
    """
    out_dir = Path(mpls.parent) if out_dir is None else out_dir

    with mpls.open("rb") as data:
        chapterdata = load_mpls(data)[:-1]

    for number, ep in enumerate(chapterdata, start=start_ep):
        startpos = ep.times[0]
        chapters = [
            f"CHAPTER{index:02d}={seconds2ts((time - startpos) / 45000.0)}\n"
            f"CHAPTER{index:02d}NAME="
            for index, time in enumerate(ep.times)
        ]
        Path(f"{out_dir}/e{number}.txt").write_text("\n".join(chapters))


if __name__ == "__main__":
    mpls2chap()



#!/usr/bin/env python
from fractions import Fraction
from pathlib import Path
from re import findall, sub

import click
from yaml import dump

from encode_utils_cli.util.source import source
from encode_utils_cli.util.timeconv import ts2f


@click.command()
@click.argument(
    "episodes",
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
@click.option("-f", "--fps", type=str, default="24000/1001")
@click.option(
    "-v", "--vid-info", is_flag=True, default=False, help="Get corresponding videos info."
)
@click.option(
    "-c",
    "--custom-layout",
    is_flag=True,
    default=False,
    help="Custom layout.",
)
def chapt2bmqpyml(
    episodes: tuple[Path],
    fps: str,
    vid_info: bool,
    custom_layout: bool,
) -> None:
    """
    Convert chapters into yaml, corresponding bookmarks and qp files.
    """
    for ep in episodes:
        chapters = ep.read_text()

        clip = source(Path(f"{ep.parents[1]}/{ep.stem}.mp4")) if vid_info else None
        fps_ = Fraction(fps if clip is None else clip.fps.numerator / clip.fps.denominator)

        names = [sub(r"[ ,]", "_", name) for name in findall(r"NAME=([^\n]+)", chapters)]
        frames = [ts2f(ts, fps_) for ts in findall(r"\d+=(\d+:\d+:\d+\.\d+)", chapters)]

        if custom_layout:
            bmk = Path(f"{ep.parents[2]}/{ep.stem}.vpy.bookmarks")
            qpf = Path(f"{ep.parents[2]}/in/{ep.stem}.qp")
            yml = Path(f"{ep.parents[2]}/chapters.yaml")
        else:
            bmk = Path(f"{ep.parent}/{ep.stem}.vpy.bookmarks")
            qpf = Path(f"{ep.parent}/{ep.stem}.qp")
            yml = Path(f"{ep.parent}/chapters.yaml")

        bmk.write_text(", ".join(f"{frame}" for frame in frames) + "\n")
        qpf.write_text("\n".join(f"{frame} I -1" for frame in frames) + "\n")

        chap = {ep.stem: dict(zip(names, frames))}
        if clip is not None:
            chap[ep.stem]["EOF"] = clip.num_frames

        with yml.open("a") as stream:
            dump(data=chap, stream=stream, sort_keys=False)



#!/usr/bin/env python
from pathlib import Path

import click

from encode_utils_cli.util.source import source


@click.command()
@click.argument(
    "vids",
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
def num_frames(vids: tuple[Path]) -> None:
    """
    Print clip num frames.
    """
    for vid in vids:
        click.echo(f"{vid.stem}.num_frames: {source(vid).num_frames}")



#!/usr/bin/env python
from pathlib import Path

import click
from schema import Regex, Schema


@click.command()
@click.argument(
    "zones_config",
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
def zones_validator(zones_config: Path) -> None:
    """
    Validate x264/x265 zones.
    """
    text = zones_config.read_text()

    targe_lines = [
        line
        for line in text.splitlines()
        if line and not line.isspace() and not line.lstrip().startswith("#")
    ]

    for line in targe_lines:
        if error_zones := [
            zone
            for zone in line.split(": ")[1].split("/")
            if not Schema(Regex(r"^\d+,\d+,b=\d\.\d+$")).is_valid(zone)
        ]:
            click.echo(f"{line} <- {error_zones}")



#!/usr/bin/env python
from pathlib import Path

from vapoursynth import VideoNode, core


def source(video: Path) -> VideoNode:
    return (
        core.lsmas.LibavSMASHSource(source=video)
        if video.suffix == ".mp4"
        else core.lsmas.LWLibavSource(source=video)
    )



#!/usr/bin/env python
from fractions import Fraction
from pathlib import PurePath
from re import search

import click
from pyperclip import copy

from encode_utils_cli.util.timeconv import ts2f


@click.command()
@click.argument(
    "screens",
    nargs=-1,
    required=True,
    type=click.Path(dir_okay=False, path_type=PurePath),
)
@click.option("-f", "--fps", type=str, default="24000/1001")
def screens2bm(screens: tuple[PurePath], fps: str) -> None:
    """
    Parse screens timestamps hh:mm:ss.xxxx into bookmark format.
    The result will be copied to the clipboard.

    \b
    >>> 00000 (00:12:34.34) 01.png
    <<< 18086
    >>> 00000 (00_00_03.34) 02.png
    <<< 80
    """
    frames = [
        f"{ts2f(ts=ts.group(1).replace('_', ':'), fps=Fraction(fps))}"
        for screen in screens
        if (ts := search(r"(\d+[:_]\d+[:_]\d+\.\d+)", screen.stem))
    ]
    bookmark = ", ".join(frames) + "\n"
    click.echo(bookmark)
    copy(bookmark)



#!/usr/bin/env python
from pathlib import Path
from re import findall, sub

import click
from yaml import safe_load


@click.command()
@click.argument(
    "episodes",
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
@click.option(
    "-c",
    "--config",
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
    help="yaml config.",
)
def re_chapters(episodes: tuple[Path], config: Path) -> None:
    """
    Replace chapters names.
    """
    names = safe_load(config.read_text())

    for ep in episodes:
        chapters = ep.read_text()
        chap_count = len(findall(r"(NAME=)", chapters))
        zero_chap = findall(r"(CHAPTER00NAME=)", chapters)
        pad = 0 if zero_chap else 1

        for i in range(chap_count):
            chapters = sub_chapter(
                chapters=chapters,
                num=i + pad,
                name=names[chap_count][i],
            )

        ep.write_text(chapters)


def sub_chapter(chapters: str, num: int, name: str) -> str:
    """
    Replace chapter name.
    """
    return sub(rf"(CHAPTER{num:02d}NAME=)(.*)", rf"\1{name}", chapters)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['encode_utils_cli', 'encode_utils_cli.util']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click>=8.0.2,<9.0.0',
 'pyperclip>=1.8.2,<2.0.0',
 'schema>=0.7.5,<0.8.0',
 'tomli>=2.0.1,<3.0.0',
 'vapoursynth>=59,<60']

entry_points = \
{'console_scripts': ['encode-utils-cli = encode_utils_cli._cli:cli']}

setup_kwargs = {
    'name': 'encode-utils-cli',
    'version': '0.0.3',
    'description': 'Encode utils collection',
    'long_description': '# encode-utils-cli\n\n> Encode utils collection\n\n[![PyPI version](https://img.shields.io/pypi/v/encode-utils-cli)](https://pypi.org/project/encode-utils-cli)\n[![CI/CD](https://github.com/DeadNews/encode-utils-cli/actions/workflows/python-vs-app.yml/badge.svg)](https://github.com/DeadNews/encode-utils-cli/actions/workflows/python-vs-app.yml)\n[![pre-commit.ci](https://results.pre-commit.ci/badge/github/DeadNews/encode-utils-cli/main.svg)](https://results.pre-commit.ci/latest/github/DeadNews/encode-utils-cli/main)\n[![codecov](https://codecov.io/gh/DeadNews/encode-utils-cli/branch/main/graph/badge.svg?token=OCZDZIYPMC)](https://codecov.io/gh/DeadNews/encode-utils-cli)\n\n## Installation\n\n```sh\npip install encode-utils-cli\n# or\npipx install encode-utils-cli\n```\n\n## CLI Reference\n\n<https://deadnews.github.io/encode-utils-cli>\n',
    'author': 'DeadNews',
    'author_email': 'uhjnnn@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/DeadNews/encode-utils-cli',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
from io import SEEK_CUR
from struct import unpack
from typing import BinaryIO, NamedTuple


class PlayList(NamedTuple):
    name: str
    times: list[int]


def load_mpls(f: BinaryIO, fix_overlap: bool = True) -> list[PlayList]:
    """
        Parse blu-ray .mpls
        https://gist.github.com/dk00/0a0634c5666cf1b8ab9f

    >>> [
    >>>     PlayList(name="00014", times=[189000000, 194469213, 225901239, 249525465, 253620806]),
    >>>     PlayList(name="00015", times=[189000000, 200779267, 223110326, 249510450, 253620806]),
    >>> ]
    """

    def int_be(data: bytes) -> int:
        funcs = {
            1: ord,
            2: lambda b: unpack(">H", b)[0],
            4: lambda b: unpack(">I", b)[0],
        }
        return funcs[len(data)](data)

    f.seek(8)
    addr_items, addr_marks = int_be(f.read(4)), int_be(f.read(4))
    f.seek(addr_items + 6)
    item_count = int_be(f.read(2))
    f.seek(2, SEEK_CUR)

    def read_item() -> PlayList:
        block_size = int_be(f.read(2))
        name = f.read(5).decode()
        f.seek(7, SEEK_CUR)
        times = [int_be(f.read(4)), int_be(f.read(4))]
        f.seek(block_size - 20, SEEK_CUR)
        return PlayList(name, times)

    items = [read_item() for _ in range(item_count)]

    f.seek(addr_marks + 4)
    mark_count = int_be(f.read(2))

    def read_mark() -> tuple[int, int]:
        f.seek(2, SEEK_CUR)
        index = int_be(f.read(2))
        time = int_be(f.read(4))
        f.seek(6, SEEK_CUR)
        return (index, time)

    for _ in range(mark_count):
        index, time = read_mark()
        if time > items[index].times[-2]:
            items[index].times.insert(-1, time)

    if fix_overlap:
        b = None
        for item in items:
            a, b = b, item.times
            if a and b[0] < a[-1] < b[-1]:
                a[-1] = b[0]
        if b is not None and len(b) > 1 and b[-1] - b[-2] < 90090:
            b.pop()

    return items



#!/usr/bin/env python
from pathlib import Path
from random import sample

import click
from vapoursynth import RGB24, VideoNode, core

from encode_utils_cli.util.source import source


@click.command()
@click.argument(
    "vids",
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
@click.option(
    "-d",
    "--out-dir",
    type=click.Path(file_okay=False, path_type=Path),
    help="Custom out dir.",
)
@click.option("-f", "--frames", type=str, help="Frames. Format: '1 2 3'.")
@click.option("-o", "--offset", type=int, default=0, help="Offset for clip.")
@click.option("-c", "--crop", type=int, default=0, help="CropRel args.")
@click.option("-p", "--drop-prop", is_flag=True, help="Delete frame prop.")
def vs_screens(
    vids: tuple[Path],
    out_dir: Path,
    frames: str,
    offset: int,
    crop: int,
    drop_prop: bool,
) -> None:
    """
    Screens via Vapoursynth.
    """
    frames = frames or " ".join([f"{i}" for i in sample(range(100, 10000), k=5)])
    click.echo(f"Requesting frames: {frames!r}")

    for vid in vids:
        out_dir = Path(vid.parent) if out_dir is None else out_dir
        out_dir.mkdir(exist_ok=True)
        save_pattern = Path(f"{out_dir}/{vid.stem} %d.png")

        clip = open_clip(vid, drop_prop=drop_prop, offset=offset, crop=crop)
        writer = core.imwri.Write(clip, "png", save_pattern)

        for frame in frames.split():
            click.echo(f"Writing: '{save_pattern}' {frame}")
            writer.get_frame(int(frame))


def open_clip(video: Path, drop_prop: bool, offset: int, crop: int) -> VideoNode:
    """
    Prepare clip.
    """
    clip = source(video)

    if drop_prop:
        clip = (
            clip.std.Setframe_prop(prop="_Matrix", delete=True)
            .std.Setframe_prop(prop="_Transfer", delete=True)
            .std.Setframe_prop(prop="_Primaries", delete=True)
        )
    clip = clip.resize.Spline36(format=RGB24, matrix_in_s="709" if clip.height >= 576 else "601")
    if offset:
        clip = clip.std.Trim(offset, clip.num_frames - 1)
    if crop:
        clip = clip.std.CropRel(top=crop, bottom=crop)

    return clip
